-- 客户管理表
CREATE TABLE `wm_af_customer` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键id',
  `dept_id` int(11) DEFAULT NULL COMMENT '部门id',
  `customer_number` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '客户id',
  `customer_name` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '客户名称',
  `customer_type` varchar(20) NOT NULL COMMENT '客户类别(0-药企、1-CSO、2-CRO、3-其他)',
  `customer_manager` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '客户经理',
  `channel_name` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '渠道名称',
  `sign_date` datetime DEFAULT NULL COMMENT '签订日期',
  `expiration_date` datetime DEFAULT NULL COMMENT '有效期截止',
  `customer_status` varchar(20) DEFAULT NULL COMMENT '客户状态:(0-客户备案、1-合同立项、2-签约 、3-上线)',
  `remark` varchar(128) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '说明',
  `contract_file` varchar(512) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '合同附件',
  `contract_name` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '合同名称',
  `create_time` datetime DEFAULT NULL COMMENT '创建时间',
  `create_user` int(11) DEFAULT NULL COMMENT '创建人',
  `update_time` datetime DEFAULT NULL COMMENT '更新时间',
  `update_user` int(11) DEFAULT NULL COMMENT '更新人',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='档案-客户管理';



-- GMV数据统计主表
CREATE TABLE `wm_dept_gmv` (
   `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键id',
   `dept_id` int(11) NOT NULL COMMENT '部门ID',
   `year` char(4) NOT NULL COMMENT '年份',
   `month` char(4) NOT NULL COMMENT '月份',
   `update_time` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '修改时间',
   PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='GMV数据统计主表';

-- GMV数据统计每日记录表
CREATE TABLE `wm_dept_gmv_record` (
  `id` int(11) NOT NULL AUTO_INCREMENT COMMENT 'id',
  `gmv_id` int(11) NOT NULL COMMENT 'GMV数据统计主表id',
  `statistics_day` char(2) NOT NULL COMMENT '统计日(1-31)',
  `gmv1` bigint(20) NOT NULL COMMENT '按积分值统计的GMV（单位：积分）',
  `gmv1_rate` int(11) NOT NULL COMMENT 'gmv1收益率（万分之）',
  `gmv2` bigint(20) NOT NULL COMMENT '按结算额统计的GMV（单位：分）',
  `gmv2_rate` int(11) NOT NULL COMMENT 'gmv2收益率（万分之）',
  `renlijia_service_charge_rate` int(11) DEFAULT NULL COMMENT '人力家服务费（万分之）',
  `income` bigint(20) NOT NULL COMMENT '要易收益（单位：分）',
  `update_time` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '修改时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='GMV数据统计每日记录表';

-- 部门表新增字段
ALTER TABLE `sys_dept`
ADD COLUMN `gmv1_rate`  int(11) NULL COMMENT 'gmv1收益率（万分之）' AFTER `dept_permissions`,
ADD COLUMN `gmv2_rate`  int(11) NULL COMMENT 'gmv2收益率（万分之）' AFTER `gmv1_rate`,
ADD COLUMN `renlijia_service_charge_rate`  int(11) NULL COMMENT '人力家服务费（万分之）' AFTER `gmv2_rate`;

-- GMV数据统计主表加GMV相关字段
ALTER TABLE `wm_dept_gmv`
ADD COLUMN `gmv1` bigint(20) NOT NULL COMMENT '按积分值统计的GMV（单位：积分）' AFTER `month`,
ADD COLUMN `gmv1_rate` int(11) NOT NULL COMMENT 'gmv1收益率（万分之）' AFTER `gmv1`,
ADD COLUMN `gmv2` bigint(20) NOT NULL COMMENT '按结算额统计的GMV（单位：分）' AFTER `gmv1_rate`,
ADD COLUMN `gmv2_rate` int(11) NOT NULL COMMENT 'gmv2收益率（万分之）' AFTER `gmv2`,
ADD COLUMN `renlijia_service_charge_rate` int(11) DEFAULT NULL COMMENT '人力家服务费（万分之）' AFTER `gmv2_rate`,
ADD COLUMN `income` bigint(20) NOT NULL COMMENT '要易收益（单位：分）' AFTER `renlijia_service_charge_rate`;
