/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.api.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 部门管理
 * </p>
 *
 * @author hnqz
 * @since 2018-01-22
 */
@Data
@ApiModel(value = "部门")
@EqualsAndHashCode(callSuper = true)
public class SysDept extends Model<SysDept> {

	private static final long serialVersionUID = 1L;

	/**
	 *
	 */
	@TableId
	@ApiModelProperty(value="")
	private Integer deptId;
	/**
	 * 部门名称
	 */
	@NotBlank(message = "部门名称不能为空")
	@ApiModelProperty(value = "部门名称")
	private String name;

	/**
	 * 排序
	 */
	@NotNull(message = "排序值不能为空")
	@ApiModelProperty(value = "排序值")
	private Integer sort;

	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间")
	private LocalDateTime createTime;

	/**
	 * 修改时间
	 */
	@ApiModelProperty(value = "修改时间")
	private LocalDateTime updateTime;
	/**
	 *
	 */
	@ApiModelProperty(value="",hidden=true)
	private Integer tenantId;
	/**
	 * 父级部门id
	 */
	@ApiModelProperty(value = "父级部门id")
	private Integer parentId;

	/**
	 * 是否删除 1：已删除 0：正常
	 */
	@TableLogic
	@ApiModelProperty(value = "删除标记,1:已删除,0:正常")
	private String delFlag;



	/**
	 *
	 */
	@ApiModelProperty(value="")
	private String deptCode;
	/**
	 * 级别
	 */
	@ApiModelProperty(value="级别")
	private Integer level;
	/**
	 * 药企名称
	 */
	@ApiModelProperty(value="药企名称")
	private String level1Name;
	/**
	 * 药企ID
	 */
	@ApiModelProperty(value="药企ID")
	private Integer level1Id;
	/**
	 * 一级代理商名称
	 */
	@ApiModelProperty(value="一级代理商名称")
	private String level2Name;
	/**
	 * 一级代理商ID
	 */
	@ApiModelProperty(value="一级代理商ID")
	private Integer level2Id;
	/**
	 * 二级代理商名称
	 */
	@ApiModelProperty(value="二级代理商名称")
	private String level3Name;
	/**
	 * 二级代理商ID
	 */
	@ApiModelProperty(value="二级代理商ID")
	private Integer level3Id;

	private String appId;
	private String appSecret;
	private String rsaPublicKey;
	private String queryUrl;
	private String subjectName;

	/**
	 * 税号
	 */
	@ApiModelProperty(value="税号")
	private String taxCode;

	/**
	 * 结算主体类型：0->人力家，1->税邦云
	 */
	@ApiModelProperty(value="结算主体类型")
	private String[] subjectType;

	/**
	 * 部门专用权限限制（字符串逗号分隔，0.导出）
	 */
	@ApiModelProperty(value="部门专用权限限制（字符串逗号分隔，0.导出）")
	private String[] deptPermissions;

	@TableField(exist = false)
	private List<String> subList;

	/**
	 * gmv1收益率（万分之）
	 */
	@ApiModelProperty(value="gmv1收益率（万分之）",hidden = true)
	private Integer gmv1Rate;

	/**
	 * gmv2收益率（万分之）
	 */
	@ApiModelProperty(value=" gmv2收益率（万分之）",hidden = true)
	private Integer gmv2Rate;

	/**
	 * 人力家服务费费率（万分之）
	 */
	@ApiModelProperty(value="人力家服务费费率（万分之）", hidden = true)
	private Integer renlijiaServiceChargeRate;

}
