/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.service.WmTaskContentService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 任务内容表
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:25
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmtaskcontent" )
@Api(value = "wmtaskcontent", tags = "任务内容表管理")
public class WmTaskContentController {

    private final  WmTaskContentService wmTaskContentService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmTaskContent 任务内容表
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
	//@PreAuthorize("@pms.hasPermission('admin_wmtaskcontent_view')" )
    public R getWmTaskContentPage(Page page, WmTaskContent wmTaskContent) {
//		OrderItem orderItem=new OrderItem();
//		orderItem.setColumn("create_time");
//		orderItem.setAsc(false);
//		page.addOrder(orderItem);

        return R.ok(wmTaskContentService.page(page, Wrappers.query(wmTaskContent)));
    }


    /**
     * 通过id查询任务内容表
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
	//@PreAuthorize("@pms.hasPermission('admin_wmtaskcontent_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmTaskContentService.getById(id));
    }

    /**
     * 新增任务内容表
     * @param wmTaskContent 任务内容表
     * @return R
     */
    @ApiOperation(value = "新增任务内容表", notes = "新增任务内容表")
    @SysLog("新增任务内容表" )
    @PostMapping
	//@PreAuthorize("@pms.hasPermission('admin_wmtaskcontent_add')" )
    public R save(@RequestBody WmTaskContent wmTaskContent) {
    	return R.ok(wmTaskContentService.save(wmTaskContent));
    }

    /**
     * 修改任务内容表
     * @param wmTaskContent 任务内容表
     * @return R
     */
    @ApiOperation(value = "修改任务内容表", notes = "修改任务内容表")
    @SysLog("修改任务内容表" )
    @PutMapping
	//@PreAuthorize("@pms.hasPermission('admin_wmtaskcontent_edit')" )
    public R updateById(@RequestBody WmTaskContent wmTaskContent) {
        return R.ok(wmTaskContentService.updateById(wmTaskContent));
    }

    /**
     * 通过id删除任务内容表
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除任务内容表", notes = "通过id删除任务内容表")
    @SysLog("通过id删除任务内容表" )
    @DeleteMapping("/{id}" )
	//@PreAuthorize("@pms.hasPermission('admin_wmtaskcontent_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmTaskContentService.removeById(id));
    }

	/**
	 * 获取药企列表
	 * @return 角色列表
	 */
	@GetMapping("/list")
	public R listWmScorePackage() {
		WmTaskContent wmDaDrugEnt=new WmTaskContent();
		return R.ok(wmTaskContentService.list(Wrappers.query(wmDaDrugEnt)));
	}

}
