/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.daemon.quartz.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmScorePackageService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 积分包
 *
 * @author pigx code generator
 * @date 2020-07-31 20:07:52
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmscorepackage" )
@Api(value = "wmscorepackage", tags = "积分包管理")
public class WmScorePackageController {

    private final  WmScorePackageService wmScorePackageService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmScorePackage 积分包
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackage_view')" )
    public R getWmScorePackagePage(Page page, WmScorePackage wmScorePackage) {
        return R.ok(wmScorePackageService.page(page, Wrappers.query(wmScorePackage)));
    }


    /**
     * 通过id查询积分包
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackage_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmScorePackageService.getById(id));
    }

    /**
     * 新增积分包
     * @param wmScorePackage 积分包
     * @return R
     */
    @ApiOperation(value = "新增积分包", notes = "新增积分包")
    @SysLog("新增积分包" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackage_add')" )
    public R save(@RequestBody WmScorePackage wmScorePackage) {
        return R.ok(wmScorePackageService.save(wmScorePackage));
    }

    /**
     * 修改积分包
     * @param wmScorePackage 积分包
     * @return R
     */
    @ApiOperation(value = "修改积分包", notes = "修改积分包")
    @SysLog("修改积分包" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackage_edit')" )
    public R updateById(@RequestBody WmScorePackage wmScorePackage) {
        return R.ok(wmScorePackageService.updateById(wmScorePackage));
    }

    /**
     * 通过id删除积分包
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除积分包", notes = "通过id删除积分包")
    @SysLog("通过id删除积分包" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('quartz_wmscorepackage_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmScorePackageService.removeById(id));
    }

}
